/**
 	- Eclipse Plug-in Developer Community for Korean -
 	http://cafe.naver.com/eclipseplugin
 	Copyright (C) 2005 Seok-Ho, Yang <seokho.yang@gmail.com>

    This file is part of Eclipse DictionaryView.

    DictionaryView is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    EclipseComics is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DictionaryView; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
    
    Linking this library statically or dynamically with other modules is
    making a combined work based on this library.  Thus, the terms and
    conditions of the GNU General Public License cover the whole
    combination.

	As a special exception, the copyright holders of this library give you
	permission to link this library with independent modules to produce an
	executable, regardless of the license terms of these independent
	modules, and to copy and distribute the resulting executable under
	terms of your choice, provided that you also meet, for each linked
	independent module, the terms and conditions of the license of that
	module.  An independent module is a module which is not derived from
	or based on this library.  If you modify this library, you may extend
	this exception to your version of the library, but you are not
	obligated to do so.  If you do not wish to do so, delete this
	exception statement from your version.
*/
package javanese.dictionary.views;


import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.browser.LocationEvent;
import org.eclipse.swt.browser.LocationListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;


/**
 * ̹  Ŭ  ̿ϱ  Դϴ.
 */
public class DictionaryView extends ViewPart {
	

	private String currentWord = "";
	private Combo word;
	private Combo type;
	private Browser browser;

	private Action actionHome;
	private Action actionBack;
	private Action actionForward;

	/**
	 * The constructor.
	 */
	public DictionaryView() {
	}

	/**
	 * This is a callback that will allow us
	 * to create the viewer and initialize it.
	 */
	public void createPartControl(Composite parent) {
		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		parent.setLayout(layout);

		Label label = new Label(parent, SWT.NONE);
		label.setText("˻ ܾ :");
		label.setLayoutData(new GridData());
		
		word = new Combo(parent, SWT.NONE);
		word.setVisibleItemCount(5);
		word.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		WordListener l = new WordListener();
		word.addSelectionListener(l);
		//word.addModifyListener(l);
		
		type = new Combo(parent, SWT.READ_ONLY);
		type.setLayoutData(new GridData());
		type.setItems(dicNames);
		type.select(0);
		
		browser = new Browser(parent, SWT.NONE);
		GridData d = new GridData(GridData.FILL_BOTH);
		d.horizontalSpan = 3;
		browser.setLayoutData(d);
		browser.addLocationListener(new LocationListener() {
			public void changing(LocationEvent event) {
				disableActions();
			}
			public void changed(LocationEvent event) {
				updateActions();
			}
		});
		
		makeActions();
		hookContextMenu();
		contributeToActionBars();
	}

	final static String[] dicNames = {
		"", "", "", "", "Ͼ"
	};
	
	final static String[] dicQuerys = {
		"http://endic.naver.com/small.naver?query=",	
		"http://eedic.naver.com/small.naver?query=",	
		"http://krdic.naver.com/small.naver?query=",	
		"http://hanja.naver.com/small.naver?query=",	
		"http://jpdic.naver.com/small.naver?query=",	
	};
	
	class WordListener implements SelectionListener, ModifyListener {
		
		public void widgetSelected(SelectionEvent e) {
			widgetDefaultSelected(e);
		}

		public void widgetDefaultSelected(SelectionEvent e) {
			searchWord(word.getText());
		}

		public void modifyText(ModifyEvent e) {
		}
		
	}

	protected void searchWord(String newWord) {
		if (currentWord==newWord) return;
		currentWord = newWord;
		// Combo Word Widget  ģ Browser ϱ 
		// Asyncϰ 
		Display.getCurrent().asyncExec(new Runnable() {
			public void run() {
				browser.setUrl(dicQuerys[type.getSelectionIndex()]+currentWord);
				if (word.indexOf(currentWord)>=0) word.remove(currentWord);
				word.add(currentWord, 0);
				word.setText("");
				if (word.getItemCount()>20) word.remove(20);
			}
		});
	}
	
	public void setDictionaryType(int type) {
		this.type.select(type);
	}
	
	public void setWord(String word) {
		searchWord(word);
	}
	
	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				DictionaryView.this.fillContextMenu(manager);
			}
		});
		Menu menu = menuMgr.createContextMenu(browser);
		browser.setMenu(menu);
		//getSite().registerContextMenu(menuMgr, browser);
	}

	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		fillLocalPullDown(bars.getMenuManager());
		fillLocalToolBar(bars.getToolBarManager());
	}

	private void fillLocalPullDown(IMenuManager manager) {
		manager.add(actionBack);
		manager.add(actionForward);
		manager.add(new Separator());
		manager.add(actionHome);
	}

	private void fillContextMenu(IMenuManager manager) {
		manager.add(actionBack);
		manager.add(actionForward);
		manager.add(new Separator());
		manager.add(actionHome);
		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}
	
	private void fillLocalToolBar(IToolBarManager manager) {
		manager.add(actionBack);
		manager.add(actionForward);
		manager.add(new Separator());
		manager.add(actionHome);
	}

	private void makeActions() {
		actionHome = new Action() {
			public void run() {
				browser.setUrl("http://endic.naver.com/small.naver?where=index");
			}
		};
		actionHome.setText("̹ ");
		actionHome.setToolTipText("̹  ʱȭ");
		actionHome.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().
			getImageDescriptor(ISharedImages.IMG_TOOL_NEW_WIZARD));
		
		actionBack = new Action() {
			public void run() {
				browser.back();
			}
		};
		actionBack.setText("ڷ");
		actionBack.setToolTipText("ڷ");
		actionBack.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().
			getImageDescriptor(ISharedImages.IMG_TOOL_BACK));
		
		actionForward = new Action() {
			public void run() {
				browser.forward();
			}
		};
		actionForward.setText("");
		actionForward.setToolTipText("");
		actionForward.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().
			getImageDescriptor(ISharedImages.IMG_TOOL_FORWARD));
		
		updateActions();
	}
	
	protected void disableActions() {
		if (actionBack!=null) actionBack.setEnabled(false);
		if (actionForward!=null) actionForward.setEnabled(false);
	}
	
	protected void updateActions() {
		if (actionBack!=null) actionBack.setEnabled(browser.isBackEnabled());
		if (actionForward!=null) actionForward.setEnabled(browser.isForwardEnabled());
	}
	
	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {
		word.setFocus();
	}
}